///////////////////////////////////////////
/// CImageData Class
///////////////////////////////////////////

#if !defined(IMAGEDATA_H__0_0_0_0_0__INCLUDED_)
#define IMAGEDATA_H__0_0_0_0_0__INCLUDED_

#if !defined(PCCLIB_API)

#if PCCLIB_EXPORTS
#define PCCLIB_API __declspec(dllexport)
#else
#define PCCLIB_API
#endif

#endif

#include	<windows.h>
#include	<time.h>

#include	"ConstantValues.h"
#include	"LutControl.h"
#include	"CameraControl.h"

class PCCLIB_API CImageData  
{
public:
	CImageData();
	virtual			~CImageData();

	BOOL			Init(int buffer_size = IMAGE_SIZE_DEF_X*IMAGE_SIZE_DEF_Y*3);
	BOOL			Exit();
	BOOL			IsInit();

	void			InitLut();
	void			SetUseLut(BOOL on);
	BOOL			GetUseLut();
	BOOL			SetLutR(int * lut);
	BOOL			SetLutG(int * lut);
	BOOL			SetLutB(int * lut);
	BOOL			GetLutR(int * lut);
	BOOL			GetLutG(int * lut);
	BOOL			GetLutB(int * lut);
	BOOL			SetLut(int * lut, int plane = LUT_PLANE_ALL);
	BOOL			GetLut(int * lut, int plane = LUT_PLANE_ALL);
	BOOL			SetLutGamma(double gamma, int plane = LUT_PLANE_ALL);
	BOOL			SetLutContrast(int contrast, int plane = LUT_PLANE_ALL);
	BOOL			SetLutBrightness(int brightness, int plane = LUT_PLANE_ALL);
	double			GetLutGamma(int plane = LUT_PLANE_ALL);
	int				GetLutContrast(int plane = LUT_PLANE_ALL);
	int				GetLutBrightness(int plane = LUT_PLANE_ALL);

	BYTE*			GetImageBuff();
	BITMAPINFO*		GetBmpInfo();
	BOOL			SetBmpInfo(BITMAPINFO* bmi);
	void			GetImageSize(int& x, int& y);
	BYTE*			GetWindowsImage(BYTE* buff = NULL, BITMAPINFO* bmi = NULL);
	WORD*			GetWindows16BitImage(WORD* buff = NULL, BITMAPINFO* bmi = NULL);
	BOOL			DrawImage(HWND hwnd, int offset_wx = 0, int offset_wy = 0, double zoom = 1.0, RECT* image_rect = NULL, BYTE* buff = NULL, BITMAPINFO* bmi = NULL);
	BOOL			Convert16to8BitImage(int offset = 8, BOOL higher_bit_clip = TRUE);	//16bit->8bit convert

	int				LoadFile(LPCTSTR filename);
	BOOL			LoadRawFile(LPCTSTR filename, int size_header,
						int width, int height, int plane,
						BOOL interleave = TRUE);
	BOOL			LoadRawwFile(LPCTSTR filename, int size_header,
						int width, int height, int plane,
						BOOL interleave = TRUE);
	BOOL			LoadRawBayerFile(LPCTSTR filename, int size_header,
						int width, int height, int camera_type,
						int chroma_level, int edge_enhance, int geometric, PBYTE pLutR, PBYTE pLutG, PBYTE pLutB,
						BOOL interleave = TRUE);
	BOOL			LoadRawwBayerFile(LPCTSTR filename, int size_header,
						int width, int height, int camera_type,
						int chroma_level, int edge_enhance, int geometric, int bitDepth, int effective_bitside,
						PWORD pLutR, PWORD pLutG, PWORD pLutB,
						BOOL interleave = TRUE);
	BOOL			SaveFile(LPCTSTR filename, int compress = 0, int comp_quality = 75, BOOL ascii = FALSE);
	BOOL			SaveBmpFile(LPCTSTR filename);
	BOOL			SaveTiffFile(LPCTSTR filename, BOOL compress = FALSE);
	BOOL			SaveTiffFileEx(LPCTSTR filename, BOOL compress = FALSE, int depth = 8);
	BOOL			SaveJpegFile(LPCTSTR filename, int quality = 75);
	BOOL			SavePnmFile(LPCTSTR filename, BOOL ascii = FALSE);
	BOOL			SaveRawFile(LPCTSTR filename);
	BOOL			SaveRawwFile(LPCTSTR filename, int bit_depth = 16, int bit_side = RAWW_EFFECTIVE_BITS_HIGHSIDE, WORD* buff = NULL);
	BOOL			SavePngFile(LPCTSTR filename, int comp_mode = PNG_COMPRESS_NORMAL, int depth = 8);
	BOOL			SaveFtifFile(LPCTSTR filename);

	BOOL			SetTiffHeaderMCDL(CAMERA_PARAMS params, CAMERA_PARAMS_EX paramsEx, BYTE* data = NULL);
	void			SetTiffHeaderIRIG(IRIGLIB_TIMECODE irig_time);
	void			SetTiffHeaderTime(tm time);

	BYTE*			ReAllocateBuffer(int buffer_size, BOOL surely = FALSE);
	WORD*			ReAllocate16BitBuffer(int buffer_size, BOOL surely = FALSE);
	BOOL			Is16BitImage() { return m_b16bitImage; }
	void			SetColorMatrix(int rr, int rg, int rb, int gr, int gg, int gb, int br, int bg, int bb);

	void			GeometricConvertion(int mode);
	void			GeometricConvertion8bit(BYTE* buff_src, BYTE* buff_des, int mode);
	void			GeometricConvertion16bit(WORD* buff_src, WORD* buff_des, int mode);
	int				GetCurrentGeometricMode()		{return m_nCurrentGeometricMode;}

protected:
	BOOL			m_bInit;

	HGLOBAL			m_gbImageBuff;
	HGLOBAL			m_gbBiData;
	BYTE*			m_pImageBuff;

	BYTE			m_pMCDLBuff[SIZE_OF_MCDL_DATA];
	BYTE*			m_pBiData;	
	BYTE*			m_pAccessBuff;
	BITMAPINFO*		m_pBmpInfo;
	BYTE*			m_pAccessBuffMCDL;

	int				m_nMaxImageWidth;
	int				m_nMaxImageHeight;
	int				m_nBufferSize;

	LPVOID			m_pIplImage1;
	LPVOID			m_pIplImage2;
	BOOL			m_bUseLut;
	HGLOBAL			m_gbLutRBuff;
	HGLOBAL			m_gbLutGBuff;
	HGLOBAL			m_gbLutBBuff;
	int*			m_lutR;	
	int*			m_lutG;	
	int*			m_lutB;

	CLutControl		m_LutControl;

	BOOL			m_bTimeValid;
	tm				m_tmTime;
	int				m_iFrameSave;

	CAMERA_PARAMS		m_cpCameraParams;
	CAMERA_PARAMS_EX	m_cpCameraParamsEx;
	BOOL			m_bTiffMcdl;
	int				m_nVideoDataMode;

	BOOL				m_bTiffIrig;
	IRIGLIB_TIMECODE	m_irigTime;

	HGLOBAL			m_gb16BitImageBuff;
	WORD*			m_p16BitImageBuff;
	int				m_n16BitBufferSize;
	BYTE*			m_p16to8bitLUT;
	BOOL			m_b16bitImage;
	LPVOID			m_pIpl16BitImage;

	HGLOBAL			m_gbTmpColorY;
	HGLOBAL			m_gbTmpColorYdash;
	short*			m_pTmpColorY;
	short*			m_pTmpColorYdash;

	int				m_nCurrentGeometricMode;

	int				m_ColorMatrixRR;
	int				m_ColorMatrixRG;
	int				m_ColorMatrixRB;
	int				m_ColorMatrixGR;
	int				m_ColorMatrixGG;
	int				m_ColorMatrixGB;
	int				m_ColorMatrixBR;
	int				m_ColorMatrixBG;
	int				m_ColorMatrixBB;

	void			BayerToRGB8to24bit(		int camera_type,
											int chroma_level,
											int edge_enhance,
											UINT width,
											UINT height,
											PBYTE src,
											PBYTE des,
											PBYTE pLutR,
											PBYTE pLutG,
											PBYTE pLutB);
	void			BayerToRGB16to48bit(	int camera_type,
											int chroma_level,
											int edge_enhance,
											UINT width,
											UINT height,
											PWORD src,
											PWORD dest,
											int bit_depth,
											int effective_bitside,
											PWORD pLutR,
											PWORD pLutG,
											PWORD pLutB);
	BOOL			Save8bitPaletteFile(	LPCTSTR filename,
											PBYTE pBuffR,
											PBYTE pBuffG,
											PBYTE pBuffB);
	BOOL			Save16bitPaletteFile(	LPCTSTR filename,
											PWORD pBuffR,
											PWORD pBuffG,
											PWORD pBuffB,
											int bit_depth);
	BOOL			Load8bitPaletteFile(	LPCTSTR filename,
											PBYTE pBuffR,
											PBYTE pBuffG,
											PBYTE pBuffB);
	BOOL			Load16bitPaletteFile(	LPCTSTR filename,
											PWORD pBuffR,
											PWORD pBuffG,
											PWORD pBuffB);

	void ImageProcess(DWORD nWidth, DWORD nHeight, void* src, void* des, int bit, int mode);

	void ImageProcess8BitGray_Rotate90(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_Rotate270(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_MirrorV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_Rotate180(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_MirrorH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_R90andMH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitGray_R90andMV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);

	void ImageProcess16BitGray_Rotate90(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_Rotate270(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_MirrorV(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_Rotate180(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_MirrorH(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_R90andMH(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitGray_R90andMV(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);

	void ImageProcess8BitColor_Rotate90(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_Rotate270(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_MirrorV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_Rotate180(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_MirrorH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_R90andMH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess8BitColor_R90andMV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);

	void ImageProcess16BitColor_Rotate90(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_Rotate270(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_MirrorV(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_Rotate180(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_MirrorH(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_R90andMH(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);
	void ImageProcess16BitColor_R90andMV(DWORD nWidth, DWORD nHeight, WORD* src, WORD* des);

	void ImageProcess32BitColor_Rotate90(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_Rotate270(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_MirrorV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_Rotate180(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_MirrorH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_R90andMH(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);
	void ImageProcess32BitColor_R90andMV(DWORD nWidth, DWORD nHeight, BYTE* src, BYTE* des);

};

#endif
